package es.ucm.fdi.grasia.faerie.broker.internal.osgi;

import java.net.URI;
import java.util.HashMap;
import java.util.UUID;

import org.apache.log4j.Logger;

import es.ucm.fdi.grasia.faerie.container.ContextContainer;
import es.ucm.fdi.grasia.faerie.core.Context;
import es.ucm.fdi.grasia.faerie.core.ContextElement;
import es.ucm.fdi.grasia.faerie.core.NotAccessibleContextException;

@SuppressWarnings({ "unchecked" })
public class ContextProxy implements Context {

	private ContextContainer contextContainer = null;
	private final UUID id = setUpId();
	private final HashMap<URI, Integer> elementsRequests = setUpElementsRequests();

	UUID setUpId() {
		return UUID.randomUUID();
	}

	HashMap<URI, Integer> setUpElementsRequests() {
		return new HashMap<URI, Integer>();
	}

	public synchronized void setContextContainer(ContextContainer contextContainer) {
		this.contextContainer = contextContainer;
	}

	@Override
	public synchronized boolean contains(URI elementId) {
		if (contextContainer != null)
			return contextContainer.contains(elementId, id);
		throw new NotAccessibleContextException("contextContainer is null");
	}

	@Override
	public synchronized <T extends ContextElement> T peek(URI elementId) {
		if (contextContainer != null)
			return (T) contextContainer.peek(elementId, id);
		throw new NotAccessibleContextException("contextContainer is null");
	}

	private synchronized void incRequests(URI elementId) {
		elementsRequests.put(elementId, countRequests(elementId) + 1);
	}

	private synchronized boolean decRequests(URI elementId) {
		Integer requests = countRequests(elementId);
		if (requests > 0)
			elementsRequests.put(elementId, --requests);
		return requests <= 0;
	}

	private synchronized void resetRequests(URI elementId) {
		elementsRequests.put(elementId, 0);
	}

	private synchronized void resetAllRequests() {
		elementsRequests.clear();
	}

	private synchronized int countRequests(URI elementId) {
		Integer requests = elementsRequests.get(elementId);
		if (requests == null)
			return 0;
		else
			return requests;
	}

	@Override
	public synchronized <T extends ContextElement> T request(T element) {
		if (contextContainer != null) {
			incRequests(element.getId());
			return contextContainer.request(element, id);
		}
		throw new NotAccessibleContextException("contextContainer is null");
	}

	/*	@Override
		public synchronized ContextExpression requestExpression(ContextExpression expression) {
			if (contextContainer != null) {
				//incRequests(element.getId());
				return contextContainer.requestExpression(expression, id);
			}
			throw new NotAccessibleContextException("contextContainer is null");
		}*/

	@Override
	public synchronized boolean release(URI elementId) {
		if (contextContainer != null) {
			if (decRequests(elementId)) {
				contextContainer.release(elementId, id);
				return true;
			}
			return false;
		}
		throw new NotAccessibleContextException("contextContainer is null");
	}

	@Override
	public synchronized void releaseAll(URI elementId) {
		if (contextContainer != null) {
			resetRequests(elementId);
			contextContainer.release(elementId, id);
		} else
			throw new NotAccessibleContextException("contextContainer is null");
	}

	@Override
	public synchronized void releaseAll() {
		if (contextContainer != null) {
			resetAllRequests();
			contextContainer.releaseAll(id);
		} else
			Logger.getLogger(this.getClass()).error("NotAccessibleContextException: contextContainer is null");
		// throw new NotAccessibleContextException("contextContainer is null");
	}
}
